/*!
CMouseSimple.cpp
 Menu Disc Demo
 
 Author: Jeroen Verbeek
 Copyright (c) Jeroen Verbeek, 2003-2004. All rights reserved.
 
 This software is provided without any guarantee.
 No responsibility taken for any problems caused by the use if this software.
 
 Anyone is authorised to use this program code and methods without warranty,
 all that is required is that you contact me (jeroen@happymakinggames.com),
 and tell me that you use it and what you use it in -
 all I want is to know is who uses it and where.
 
 I will not ask for payment in any way: this code is free.
 If you encounter a bug with the code, please tell me about it.
 If you have a bug-fix for the code please provide me with it.
 If you think of an improvement please tell me about it.
 You are not required to provide me with bug reports, bug fixes or improvements.
 If you require me to perform any work related to this code or
 not related to this code, please contact me: I am always open to offers.
 
 Pie menus are in no way my original idea, I stumbled on the idea a few years ago,
 specifically Don Hopkins' http://catalog.com/hopkins/piemenus/piemenus-hopkins.html.
 I am a big fan of circular menus (disc menus).
 I am of the opinion that they provide the quickest method of selecting an action
 from a structured tree of actions. A menu of actions organised following this method
 will, in time, allow almost "blind selection" of actions. Each action selection
 requires the same distance of mouse movement, just in a slightly diferent angle.
 In my opinion this seems to work best with menu discs of no more than 8 items.
 There is plenty of research to be found through the internet regarding the 
 advantages of circular menus, for example: muscle memory.
 Some websites I recommend you look at are:
 
 http://catalog.com/hopkins/piemenus/piemenus-hopkins.html
 
 http://www.piemenus.com/
 
 http://www.sm.luth.se/~david/carl/www/argonaut.html
 
 Contact: Jeroen Verbeek, jeroen@happymakinggames.com, Sydney NSW Australia
 */
/*-----------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include <time.h>
#include <GLut/glut.h>
#include "CMouseSimple.h"

 
/*-----------------------------------------------------------------------------------------------*/
/*!
\brief Utility function: Convert a pixel coord to a GL coord within a window/screen.

 <b>NOTE:</b> This function will move to a general purpose collection of (mostly) math related functions.
 This is not part of the CMouseSimple class, and is not declared in the header file.
 
 \param	pixel			The pixel to convert to GL space (-1.0f to 1.0f).
 \param halfdimension	The target window/screen width in pixels divided by half.
 */
/*-----------------------------------------------------------------------------------------------*/
#pragma warning(This function will be moved to a general purpose collection of (mostly) math functions!)
float ConvertPixelToFloat(int pixel, int halfdimension)
{
	return ((float)(pixel - halfdimension) / halfdimension);
}


/*-----------------------------------------------------------------------------------------------*/
/*!
\brief Utility function: Convert a float coord to a pixel coord within a window/screen.

 <b>NOTE:</b> This function will move to a general purpose collection of (mostly) math related functions.
 This is not part of the CMouseSimple class, and is not declared in the header file.
 
 \param	coord			The GL coord to convert to window/screen space.
 \param halfdimension	The target window/screen width in pixels divided by half.
 */
/*-----------------------------------------------------------------------------------------------*/
#pragma warning(This function will be moved to a general purpose collection of (mostly) math functions!)
int ConvertFloatToPixel(float coord, int halfdimension)
{
	return (int)((coord * halfdimension) + halfdimension);
}





/*-----------------------------------------------------------------------------------------------*/
/*!
A global CMouseSimple object for access by all.
 Needs to be created at program "start" befoire use.
 */
/*-----------------------------------------------------------------------------------------------*/
CMouseSimple *g_pMouseSimple = 0;


/*-----------------------------------------------------------------------------------------------*/
/*!
Create a simple mouse object. Query GLUT for the number of buttons on the mouse and allocate button states.
 */
/*-----------------------------------------------------------------------------------------------*/
CMouseSimple::CMouseSimple()
{
	//! Determine number of buttons, and allocate bool flags for these.
	
	/*! Fix for Mac single button: Add 1 to the reported number of 
		mouse buttons so that the CTRL key correctly gives a mouse button 2.
	*/
	m_NumButtons = glutDeviceGet(GLUT_NUM_MOUSE_BUTTONS) + 1;
	
	m_DragTime = new time_t[m_NumButtons];
	m_Buttons = new bool[m_NumButtons];
	
	//! Set Default screen or window size.
	SetScreenSize(800, 600);
	SetLimitRegion(0, 0, 800, 600);
	SetMouseXY(0, 0);

	//! Set all buttons to off now.
	for(int i = 0; i < m_NumButtons; i++)
	{
		m_Buttons[i] = false;
		m_DragTime[i] = 0;
	}
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Destructor.
 */
/*-----------------------------------------------------------------------------------------------*/
CMouseSimple::~CMouseSimple()
{
	delete m_Buttons;
	delete m_DragTime;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Get a button's state.

 \param button	Number of the button to get.
 
 \return Boolean state of the button.
 */
/*-----------------------------------------------------------------------------------------------*/
bool CMouseSimple::GetButton(int button)
{
	if(button >= 0 && button < m_NumButtons)
		return m_Buttons[button];
	return false;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Set a button's state.
 
 \param button	Number of the button to set.
 \param state	State to set the button to.
 
 \return Boolean old state of the button.
 */
/*-----------------------------------------------------------------------------------------------*/
bool CMouseSimple::SetButton(int button, bool state)
{
	if(button>=0 && button < m_NumButtons)
	{
		time_t tt;
		bool oldstate = m_Buttons[button];
		m_Buttons[button] = state;
		if(state)
		{
			time(&tt);
			m_DragTime[button] = tt;
		}
		else
			m_DragTime[button] = 0;
		return oldstate;
	}
	return false;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Set the mouse position (data only).
 
 \param x	X coordinate of the mouse.
 \param y	Y coordinate of the mouse.
*/
/*-----------------------------------------------------------------------------------------------*/
void CMouseSimple::SetMouseXY(int x, int y)
{
	m_fX = ConvertPixelToFloat(x, m_iDisplayWidthHalf);
	m_fY = -ConvertPixelToFloat(y, m_iDisplayHeightHalf);
	m_X = x;
	m_Y = y;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Set the screen size, this <b>MUST ABSOLUTELY BE DONE BEFORE THE MOUSE OBJECT IS USED!</b>.
 
 \param w	Width of the display.
 \param h	Height of the display.
 */
/*-----------------------------------------------------------------------------------------------*/
void CMouseSimple::SetScreenSize(int w, int h)
{
	m_iDisplayWidth = w;
	m_iDisplayHeight = h;
	m_iDisplayWidthHalf = w / 2;
	m_iDisplayHeightHalf = h / 2;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Check if drag has occured.
 
 \param button	Number of the button to check.
 
 \return Integer The time the button has been down.
 */
/*-----------------------------------------------------------------------------------------------*/
int CMouseSimple::GetDragTime(int button)
{
	if(button >= 0 && button < m_NumButtons)
	{
		time_t diff;
		time(&diff);
		diff -= m_DragTime[button];
		return (int)diff;
	}
	return 0;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Check if a button has been clicked.
 
 \param button	Number of the button to check.
 
 \return Boolean The clicked state of the ckecked button. If button is wrong false is returned.
 */
/*-----------------------------------------------------------------------------------------------*/
bool CMouseSimple::ButtonClick(int button)
{
	if(button >= 0)// && button < m_NumButtons)
	{
		time_t diff;
		time(&diff);
		diff -= m_DragTime[button];
		if(diff < MINIMUM_DRAG_TIME)
			return true;
	}
	return false;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Reset a button's drag timer.
 
 \param button	Number of the button to reset.
*/
/*-----------------------------------------------------------------------------------------------*/
void CMouseSimple::UnClickButton(int button)
{
	if(button >= 0 && button < m_NumButtons)
		m_DragTime[button] = 0;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Set the limited region directly, using ints. 
 
 \param	x1	Left edge
 \param	y1	Top edge.
 \param	x2	Right edge.
 \param	y2	Bottom edge.
 */
/*-----------------------------------------------------------------------------------------------*/
void CMouseSimple::SetLimitRegion(int x1, int y1, int x2, int y2)
{
	m_Limit[0] = x1;
	m_Limit[1] = y1;
	m_Limit[2] = x2;
	m_Limit[3] = y2;
	
}


/*-----------------------------------------------------------------------------------------------*/
/*!
"Clip" the mouse to inside the limit region.
 
 \return Boolean true is limited not limited is false.
 */
/*-----------------------------------------------------------------------------------------------*/
bool CMouseSimple::LimitMouseToRegion(void)
{
	bool limited = false;
	if(m_X < m_Limit[0])
	{
		m_X = m_Limit[0]; 
		limited = true;
	}
	else
	{
		if(m_X > m_Limit[2])
		{
			m_X = m_Limit[2]; 
			limited = true;
		}
	}
	if(m_Y < m_Limit[1])
	{	
		m_Y = m_Limit[1]; 
		limited = true;
	}
	else
	{
		if(m_Y > m_Limit[3])
		{	
			m_Y = m_Limit[3]; 
			limited = true;
		}
	}
	glutWarpPointer(m_X, m_Y);
	SetMouseXY(m_X,m_Y);
	return limited;
}


//END